@@ -24,6 +24,7 @@
 
 #include "xsane.h"
 #include "xsane-back-gtk.h"
+#include "xsane-front-gtk.h"
 #include "xsane-preferences.h"
 #include "xsane-preview.h"
 #include "xsane-save.h"
@@ -45,7 +46,7 @@
 int xsane_parse_options(char *options, char *argv[]);
 void xsane_get_bounds(const SANE_Option_Descriptor *opt, double *minp, double *maxp);
 double xsane_find_best_resolution(int well_known_option, double dpi);
-int xsane_set_resolution(int well_known_option, double resolution);
+double xsane_set_resolution(int well_known_option, double resolution);
 void xsane_set_all_resolutions(void);
 void xsane_define_maximum_output_size();
 void xsane_close_dialog_callback(GtkWidget *widget, gpointer data);
@@ -92,7 +93,7 @@ int xsane_parse_options(char *options, char *argv[])
  int optpos = 0;
  int bufpos = 0;
  int arg    = 0;
- char buf[256];
+ char buf[TEXTBUFSIZE];
  
   DBG(DBG_proc, "xsane_parse_options\n");
  
@@ -170,11 +171,11 @@ void xsane_get_bounds(const SANE_Option_Descriptor *opt, double *minp, double *m
   {
     if (min > -INF && min < INF)
     {
-      min = SANE_UNFIX (min);
+      min = SANE_UNFIX(min);
     }
     if (max > -INF && max < INF)
     {
-      max = SANE_UNFIX (max);
+      max = SANE_UNFIX(max);
     }
   }
   *minp = min;
@@ -292,7 +293,7 @@ double xsane_find_best_resolution(int well_known_option, double dpi)
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-int xsane_set_resolution(int well_known_option, double resolution)
+double xsane_set_resolution(int well_known_option, double resolution)
 {
  const SANE_Option_Descriptor *opt;
  double bestdpi;
@@ -304,12 +305,12 @@ int xsane_set_resolution(int well_known_option, double resolution)
 
   if (!opt)
   {
-    return -1; /* option does not exits */
+    return -1.0; /* option does not exits */
   }
 
   if (!SANE_OPTION_IS_ACTIVE(opt->cap))
   {
-    return -1; /* option is not active */
+    return -1.0; /* option is not active */
   }
 
   bestdpi = xsane_find_best_resolution(well_known_option, resolution);
@@ -317,7 +318,7 @@ int xsane_set_resolution(int well_known_option, double resolution)
   if (bestdpi < 0)
   {
      DBG(DBG_error, "set_resolution: %s\n", ERR_FAILED_SET_RESOLUTION);
-    return -1;
+    return -1.0;
   }
 
   switch (opt->type)
@@ -332,14 +333,14 @@ int xsane_set_resolution(int well_known_option, double resolution)
 
     default:
      DBG(DBG_error, "set_resolution: %s %d\n", ERR_UNKNOWN_TYPE, opt->type);
-    return 1; /* error */
+    return -1.0; /* error */
   }
 
   /* it makes problems to call xsane_back_gtk_set_option. This would allow a */
   /* panel_rebuild that can mess up a lot at this place*/
   xsane_control_option(xsane.dev, well_known_option, SANE_ACTION_SET_VALUE, &dpi, 0);
 
- return 0; /* everything is ok */
+ return (bestdpi); /* everything is ok */
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -347,18 +348,32 @@ int xsane_set_resolution(int well_known_option, double resolution)
 void xsane_set_all_resolutions(void)
 {
  int printer_resolution;
+ double new_resolution;
 
   DBG(DBG_proc, "xsane_set_all_resolutions\n");
 
-  if (xsane_set_resolution(xsane.well_known.dpi_y, xsane.resolution_y)) /* set y resolution if possible */
+  new_resolution = xsane_set_resolution(xsane.well_known.dpi_y, xsane.resolution_y); /* set y resolution if possible */
+  if (new_resolution < 0) /* set y resolution not possible */
   {
-    xsane_set_resolution(xsane.well_known.dpi, xsane.resolution); /* set common resolution if necessary */
-    xsane.resolution_x = xsane.resolution;
-    xsane.resolution_y = xsane.resolution;
+    new_resolution = xsane_set_resolution(xsane.well_known.dpi, xsane.resolution); /* set common resolution if necessary */
+    if (new_resolution > 0)
+    {
+      xsane.resolution   = new_resolution;
+      xsane.resolution_x = new_resolution;
+      xsane.resolution_y = new_resolution;
+    }
+    else
+    {
+      xsane.resolution   = 72.0;
+      xsane.resolution_x = 72.0;
+      xsane.resolution_y = 72.0;
+    }
   }
-  else
+  else /* we were able to set y resolution */
   {
-    xsane_set_resolution(xsane.well_known.dpi_x, xsane.resolution_x); /* set x resolution if possible */
+    xsane.resolution_y = new_resolution;
+    new_resolution = xsane_set_resolution(xsane.well_known.dpi_x, xsane.resolution_x); /* set x resolution if possible */
+    xsane.resolution_x = new_resolution;
   }
 
   switch (xsane.param.format)
@@ -382,10 +397,6 @@ void xsane_set_all_resolutions(void)
       printer_resolution = preferences.printer[preferences.printernr]->color_resolution;
      break;
   }
-
-  xsane.zoom   = xsane.resolution   / printer_resolution;
-  xsane.zoom_x = xsane.resolution_x / printer_resolution;
-  xsane.zoom_y = xsane.resolution_y / printer_resolution; 
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -400,7 +411,7 @@ void xsane_define_maximum_output_size()
 
   if ( (opt) && (opt->unit== SANE_UNIT_MM) )
   {
-    switch(xsane.xsane_mode)
+    switch(preferences.xsane_mode)
     {
       case XSANE_SAVE:
 
@@ -418,15 +429,15 @@ void xsane_define_maximum_output_size()
         if (preferences.paper_orientation >= 8) /* rotate: landscape */
         {
           preview_set_maximum_output_size(xsane.preview,
-                                          preferences.printer[preferences.printernr]->height / xsane.zoom_y,
-                                          preferences.printer[preferences.printernr]->width  / xsane.zoom_x,
+                                          preferences.printer[preferences.printernr]->height / xsane.zoom,
+                                          preferences.printer[preferences.printernr]->width  / xsane.zoom,
                                           preferences.paper_orientation);
         }
         else /* do not rotate: portrait */
         {
           preview_set_maximum_output_size(xsane.preview,
-                                          preferences.printer[preferences.printernr]->width  / xsane.zoom_x,
-                                          preferences.printer[preferences.printernr]->height / xsane.zoom_y,
+                                          preferences.printer[preferences.printernr]->width  / xsane.zoom,
+                                          preferences.printer[preferences.printernr]->height / xsane.zoom,
                                           preferences.paper_orientation);
         }
        break;
@@ -435,7 +446,7 @@ void xsane_define_maximum_output_size()
         preview_set_maximum_output_size(xsane.preview, preferences.fax_width, preferences.fax_height, 0);
        break;
 
-      case XSANE_MAIL:
+      case XSANE_EMAIL:
         preview_set_maximum_output_size(xsane.preview, INF, INF, 0);
        break;
 
@@ -1047,7 +1058,7 @@ void xsane_option_menu_new_with_pixmap(GdkWindow *window, GtkBox *parent, const
 
 static void xsane_range_display_value_right_callback(GtkAdjustment *adjust, gpointer data)
 {
- gchar buf[256];
+ gchar buf[TEXTBUFSIZE];
  int digits = (int) data;
  GtkLabel *label;
 
@@ -1249,8 +1260,8 @@ void xsane_range_new_with_pixmap(GdkWindow *window, GtkBox *parent, const char *
 
 static void xsane_browse_filename_callback(GtkWidget *widget, gpointer data)
 {
- char filename[1024];
- char windowname[256];
+ char filename[PATH_MAX];
+ char windowname[TEXTBUFSIZE];
 
   DBG(DBG_proc, "xsane_browse_filename_callback\n");
 
@@ -1333,7 +1344,7 @@ static void xsane_filetype_callback(GtkWidget *widget, gpointer data)
     {
       if ( (!strcasecmp(extension, ".pnm"))  || (!strcasecmp(extension, ".raw"))
         || (!strcasecmp(extension, ".png"))  || (!strcasecmp(extension, ".ps"))
-        || (!strcasecmp(extension, ".rgba"))
+        || (!strcasecmp(extension, ".pdf"))  || (!strcasecmp(extension, ".rgba"))
         || (!strcasecmp(extension, ".tiff")) || (!strcasecmp(extension, ".tif"))
         || (!strcasecmp(extension, ".text")) || (!strcasecmp(extension, ".txt"))
         || (!strcasecmp(extension, ".jpg"))  || (!strcasecmp(extension, ".jpeg"))
@@ -1390,6 +1401,9 @@ void xsane_outputfilename_new(GtkWidget *vbox)
  GtkWidget *xsane_filename_counter_step_menu;
  GtkWidget *xsane_filename_counter_step_item;
  GtkWidget *xsane_label;
+ GtkWidget *pixmapwidget;
+ GdkBitmap *mask;
+ GdkPixmap *pixmap;
  gchar buf[200];
  int i,j;
  int select_item = 0;
@@ -1426,13 +1440,15 @@ void xsane_outputfilename_new(GtkWidget *vbox)
 
   hbox = gtk_hbox_new(FALSE, 2);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
 
   /* filename counter step */
- 
-  xsane_label = gtk_label_new(TEXT_FILENAME_COUNTER_STEP);
-  gtk_box_pack_start(GTK_BOX(hbox), xsane_label, FALSE, FALSE, 2);
-  gtk_widget_show(xsane_label);
+
+  pixmap = gdk_pixmap_create_from_xpm_d(xsane.xsane_window->window, &mask, xsane.bg_trans, (gchar **) step_xpm);
+  pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
+  gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 2);
+  gtk_widget_show(pixmapwidget);
+  gdk_drawable_unref(pixmap);
  
   xsane_filename_counter_step_option_menu = gtk_option_menu_new();
   xsane_back_gtk_set_tooltip(xsane.tooltips, xsane_filename_counter_step_option_menu, DESC_FILENAME_COUNTER_STEP);
@@ -1683,7 +1699,7 @@ int xsane_identify_output_format(char *filename, char *filetype, char **ext)
 
   DBG(DBG_proc, "xsane_identify_output_format\n");
 
-  if ((filetype) && (*filetype))
+  if ((filetype) && (*filetype) && (!xsane.force_filename))
   {
     extension = filetype+1; /* go to filetype, skip leading dot */
   }
@@ -1734,6 +1750,10 @@ int xsane_identify_output_format(char *filename, char *filetype, char **ext)
     {
       output_format = XSANE_PS;
     }
+    else if (!strcasecmp(extension, "pdf"))
+    {
+      output_format = XSANE_PDF;
+    }
 #ifdef HAVE_LIBTIFF
     else if ( (!strcasecmp(extension, "tif")) || (!strcasecmp(extension, "tiff")) )
     {
@@ -1768,7 +1788,7 @@ int xsane_identify_output_format(char *filename, char *filetype, char **ext)
 void xsane_change_working_directory(void)
 {
  char filename[PATH_MAX];
- char windowname[256];
+ char windowname[TEXTBUFSIZE];
 
   DBG(DBG_proc, "xsane_change_working_directory\n");
 
@@ -1780,7 +1800,7 @@ void xsane_change_working_directory(void)
     xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, TRUE, FALSE, TRUE, FALSE);
     if (chdir(filename))
     {
-     char buf[256];
+     char buf[TEXTBUFSIZE];
 
       snprintf(buf, sizeof(buf), "%s %s (%s).", ERR_CHANGE_WORKING_DIR, filename, strerror(errno));
       xsane_back_gtk_error(buf, TRUE);
@@ -1887,7 +1907,7 @@ int xsane_display_eula(int ask_for_accept)
                              XSANE_COPYRIGHT_SIGN, XSANE_COPYRIGHT_TXT,
                              TEXT_EULA,
                              TEXT_HOMEPAGE, XSANE_HOMEPAGE,
-                             TEXT_EMAIL, XSANE_EMAIL);
+                             TEXT_EMAIL_ADR, XSANE_EMAIL_ADR);
 
   label = gtk_label_new(buf);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
@@ -2131,7 +2151,7 @@ void xsane_display_gpl(void)
                              XSANE_COPYRIGHT_SIGN, XSANE_COPYRIGHT_TXT,
                              TEXT_GPL,
                              TEXT_HOMEPAGE, XSANE_HOMEPAGE,
-                             TEXT_EMAIL, XSANE_EMAIL);
+                             TEXT_EMAIL_ADR, XSANE_EMAIL_ADR);
 
   label = gtk_label_new(buf);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
@@ -2338,7 +2358,7 @@ int xsane_front_gtk_getname_dialog(const char *dialog_title, const char *desc_te
  GtkWidget *button;
  GtkWidget *vbox, *hbox;
  GtkAccelGroup *accelerator_group;
- char buf[256];
+ char buf[TEXTBUFSIZE];
                                                                                                                                  
   DBG(DBG_proc, "xsane_getname_dialog, oldname = %s\n", oldname);
                                                                                                                                  
@@ -2423,24 +2443,105 @@ int xsane_front_gtk_getname_dialog(const char *dialog_title, const char *desc_te
 }
                                                                                                                                  
 /* ---------------------------------------------------------------------------------------------------------------------- */
-#ifdef XSANE_ACTIVATE_MAIL
-void xsane_front_gtk_mail_project_update_lockfile_status()
+
+void xsane_front_gtk_list_entries_swap(GtkWidget *list_item_1, GtkWidget *list_item_2)
+{
+ char *page1;
+ char *page2;
+ char *type1;
+ char *type2;
+
+  DBG(DBG_proc, "xsane_front_gtk_list_entries_swap\n");
+
+  page1 = (char *) gtk_object_get_data(GTK_OBJECT(list_item_1), "list_item_data");
+  type1 = (char *) gtk_object_get_data(GTK_OBJECT(list_item_1), "list_item_type");
+  page2 = (char *) gtk_object_get_data(GTK_OBJECT(list_item_2), "list_item_data");
+  type2 = (char *) gtk_object_get_data(GTK_OBJECT(list_item_2), "list_item_type");
+
+  gtk_label_set(GTK_LABEL(gtk_container_children(GTK_CONTAINER(list_item_1))->data), page2);
+  gtk_label_set(GTK_LABEL(gtk_container_children(GTK_CONTAINER(list_item_2))->data), page1);
+  gtk_object_set_data(GTK_OBJECT(list_item_1), "list_item_data", page2);
+  gtk_object_set_data(GTK_OBJECT(list_item_1), "list_item_type", type2);
+  gtk_object_set_data(GTK_OBJECT(list_item_2), "list_item_data", page1);
+  gtk_object_set_data(GTK_OBJECT(list_item_2), "list_item_type", type1);
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+void xsane_front_gtk_add_process_to_list(pid_t pid)
+{
+ XsaneChildprocess *newprocess;
+ 
+  DBG(DBG_proc, "xsane_front_gtk_add_process_to_list(%d)\n", pid);
+ 
+  newprocess = malloc(sizeof(XsaneChildprocess));
+  newprocess->pid = pid;
+  newprocess->next = xsane.childprocess_list;
+  xsane.childprocess_list = newprocess;
+} 
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+int xsane_front_gtk_option_defined(char *string)
+{
+  if (string)
+  {
+    while (*string == ' ') /* skip spaces */
+    {
+      string++;
+    }
+    if (*string != 0)
+    {
+      return 1;
+    }
+  }
+  return 0;
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+#ifdef XSANE_ACTIVATE_EMAIL
+void xsane_front_gtk_email_project_update_lockfile_status()
 {
  FILE *lockfile;
  char filename[PATH_MAX];
 
-  snprintf(filename, sizeof(filename), "%s/lockfile", preferences.mail_project);
+  snprintf(filename, sizeof(filename), "%s/lockfile", preferences.email_project);
   lockfile = fopen(filename, "wb");
 
   if (lockfile)
   {
-    fprintf(lockfile, "%s\n", xsane.mail_status); /* first line is status of mail */
-    fprintf(lockfile, "%3d\n", (int) (xsane.mail_progress_val * 100));
+    fprintf(lockfile, "%s\n", xsane.email_status); /* first line is status of mail */
+    fprintf(lockfile, "%3d\n", (int) (xsane.email_progress_val * 100));
   }
 
   fclose(lockfile);
 }
 #endif
 /* ---------------------------------------------------------------------------------------------------------------------- */
+
+void xsane_project_dialog_close()
+{
+  DBG(DBG_proc, "xsane_project_dialog_close\n");
+                                                                                
+  if (xsane.project_dialog == NULL)
+  {
+    return;
+  }
+
+  if (xsane.project_dialog)
+  {
+    xsane_window_get_position(xsane.project_dialog, &xsane.project_dialog_posx, &xsane.project_dialog_posy);
+    gtk_window_move(GTK_WINDOW(xsane.project_dialog), xsane.project_dialog_posx, xsane.project_dialog_posy);
+  }
+                                                                                
+  gtk_widget_destroy(xsane.project_dialog);
+                                                                                
+  xsane.project_dialog = NULL;
+  xsane.project_list = NULL;
+  xsane.project_progress_bar = NULL;
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
                                                                                                                   
                                                                                                                                  
+
