@@ -3,7 +3,7 @@
    xsane-scan.c
 
    Oliver Rauch <Oliver.Rauch@rauch-domain.de>
-   Copyright (C) 1998-2004 Oliver Rauch
+   Copyright (C) 1998-2005 Oliver Rauch
    This file is part of the XSANE package.
 
    This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,9 @@
 #include "xsane-save.h"
 #include "xsane-gamma.h"
 #include "xsane-setup.h"
+#include "xsane-multipage-project.h"
+#include "xsane-fax-project.h"
+#include "xsane-email-project.h"
 
 #ifdef HAVE_LIBPNG
 #ifdef HAVE_LIBZ
@@ -50,7 +53,7 @@ static int xsane_test_multi_scan(void);
 void xsane_scan_done(SANE_Status status);
 void xsane_cancel(void);
 static void xsane_start_scan(void);
-void xsane_scan_dialog(void);
+gint xsane_scan_dialog(gpointer *data);
 static void xsane_create_internal_gamma_tables(void);
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -83,8 +86,6 @@ static int xsane_generate_dummy_filename(int conversion_level)
 
   if ( (xsane.mode == XSANE_GIMP_EXTENSION) ||
        (xsane.xsane_mode == XSANE_COPY) ||
-       (xsane.xsane_mode == XSANE_FAX) ||
-       (xsane.xsane_mode == XSANE_MAIL) ||
        (xsane.xsane_mode == XSANE_VIEWER) ||
        ( (xsane.xsane_mode == XSANE_SAVE)  &&
          (xsane.xsane_output_format != XSANE_PNM) &&
@@ -95,12 +96,33 @@ static int xsane_generate_dummy_filename(int conversion_level)
 
   if (tempfile) /* save to temporary file */
   {
-    xsane_back_gtk_make_path(sizeof(filename), filename, 0, 0, "conversion-", xsane.dev_name, ".ppm", XSANE_PATH_TMP);
+    xsane_back_gtk_make_path(sizeof(filename), filename, 0, 0, "xsane-conversion-", xsane.dev_name, ".ppm", XSANE_PATH_TMP);
     xsane.dummy_filename = strdup(filename);
     DBG(DBG_info, "xsane.dummy_filename = %s\n", xsane.dummy_filename);
 
    return TRUE;
   }
+  else if (xsane.xsane_mode == XSANE_MULTIPAGE) /* no conversion following, save directly to the selected filename */
+  {
+    xsane.dummy_filename = strdup(xsane.multipage_filename);
+    DBG(DBG_info, "xsane.dummy_filename = %s\n", xsane.dummy_filename);
+
+   return FALSE;
+  }
+  else if (xsane.xsane_mode == XSANE_FAX) /* no conversion following, save directly to the selected filename */
+  {
+    xsane.dummy_filename = strdup(xsane.fax_filename);
+    DBG(DBG_info, "xsane.dummy_filename = %s\n", xsane.dummy_filename);
+
+   return FALSE;
+  }
+  else if (xsane.xsane_mode == XSANE_EMAIL) /* no conversion following, save directly to the selected filename */
+  {
+    xsane.dummy_filename = strdup(xsane.email_filename);
+    DBG(DBG_info, "xsane.dummy_filename = %s\n", xsane.dummy_filename);
+
+   return FALSE;
+  }
   else /* no conversion following, save directly to the selected filename */
   {
     xsane.dummy_filename = strdup(xsane.output_filename);
@@ -118,7 +140,7 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
  SANE_Status status;
  SANE_Int len;
  int i, j;
- char buf[255];
+ char buf[TEXTBUFSIZE];
 
   DBG(DBG_proc, "xsane_read_image_data\n");
 
@@ -458,7 +480,7 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
    guint16 *buf16ptr;
    unsigned char *buf8 = (unsigned char *) buf16;
    unsigned char *buf8ptr;
-   char buf[255];
+   char buf[TEXTBUFSIZE];
 
     DBG(DBG_info, "depth = 16 bit\n");
 
@@ -884,36 +906,26 @@ static RETSIGTYPE xsane_sigpipe_handler(int signal)
 
 static int xsane_test_multi_scan(void)
 {
- char *set;
- SANE_Status status;
- const SANE_Option_Descriptor *opt;
-
   DBG(DBG_proc, "xsane_test_multi_scan\n");
 
-  opt = xsane_get_option_descriptor(xsane.dev, xsane.well_known.scansource);
-  if (opt)
+  if (xsane.mode == XSANE_GIMP_EXTENSION)
   {
-    if (SANE_OPTION_IS_ACTIVE(opt->cap))
-    {
-      if (opt->constraint_type == SANE_CONSTRAINT_STRING_LIST)
-      {
-        set = malloc(opt->size);
-        status = xsane_control_option(xsane.dev, xsane.well_known.scansource, SANE_ACTION_GET_VALUE, set, 0);
+    return FALSE;
+  }
 
-        if (status == SANE_STATUS_GOOD)
-        {
-          if (xsane.adf_scansource)
-          {
-            if (!strcmp(set, xsane.adf_scansource))
-            {
-              free(set);
-              return TRUE;
-            }
-          }
-        }
-        free(set);
-      }
-    }
+  if (xsane.batch_loop)
+  {
+    return FALSE;
+  }
+
+  if (xsane.adf_page_counter+1 < preferences.adf_pages_max)
+  {
+    return TRUE;
+  }
+  else
+  {
+    xsane.adf_page_counter = 0;
+    return FALSE;
   }
 
 #if 0 /* this is planned for the next sane-standard */
@@ -951,7 +963,7 @@ static int xsane_reduce_to_lineart()
     /* no temporary file */
     if (xsane_create_secure_file(xsane.dummy_filename)) /* remove possibly existing symbolic links for security */
     {
-     char buf[256];
+     char buf[TEXTBUFSIZE];
 
       snprintf(buf, sizeof(buf), "%s %s %s\n", ERR_DURING_SAVE, ERR_CREATE_SECURE_FILE, xsane.dummy_filename);
       xsane_back_gtk_error(buf, TRUE);
@@ -961,7 +973,7 @@ static int xsane_reduce_to_lineart()
 
   if (!abort)
   {
-    abort = xsane_save_image_as_lineart(old_dummy_filename, xsane.dummy_filename, xsane.progress_bar, &xsane.cancel_save);
+    abort = xsane_save_image_as_lineart(xsane.dummy_filename, old_dummy_filename, xsane.progress_bar, &xsane.cancel_save);
   }
 
   if (abort)
@@ -1046,31 +1058,32 @@ void xsane_scan_done(SANE_Status status)
       DBG(DBG_info, "correcting image height to %d lines\n", pixel_height);
       xsane.param.lines = pixel_height;
 
-      image_info.image_width      = xsane.param.pixels_per_line;
-      image_info.image_height     = pixel_height;
-      image_info.depth            = xsane.depth;
-      image_info.colors           = xsane.xsane_colors;
+      image_info.image_width       = xsane.param.pixels_per_line;
+      image_info.image_height      = pixel_height;
+      image_info.depth             = xsane.depth;
+      image_info.colors            = xsane.xsane_colors;
 
-      image_info.resolution_x     = xsane.resolution_x;
-      image_info.resolution_y     = xsane.resolution_y;
+      image_info.resolution_x      = xsane.resolution_x;
+      image_info.resolution_y      = xsane.resolution_y;
 
-      image_info.gamma            = xsane.gamma;
-      image_info.gamma_red        = xsane.gamma_red;
-      image_info.gamma_green      = xsane.gamma_green;
-      image_info.gamma_blue       = xsane.gamma_blue;
+      image_info.gamma             = xsane.gamma;
+      image_info.gamma_red         = xsane.gamma_red;
+      image_info.gamma_green       = xsane.gamma_green;
+      image_info.gamma_blue        = xsane.gamma_blue;
 
-      image_info.brightness       = xsane.brightness;
-      image_info.brightness_red   = xsane.brightness_red;
-      image_info.brightness_green = xsane.brightness_green;
-      image_info.brightness_blue  = xsane.brightness_blue;
+      image_info.brightness        = xsane.brightness;
+      image_info.brightness_red    = xsane.brightness_red;
+      image_info.brightness_green  = xsane.brightness_green;
+      image_info.brightness_blue   = xsane.brightness_blue;
  
-      image_info.contrast         = xsane.contrast;
-      image_info.contrast_red     = xsane.contrast_red;
-      image_info.contrast_green   = xsane.contrast_green;
-      image_info.contrast_blue    = xsane.contrast_blue;
+      image_info.contrast          = xsane.contrast;
+      image_info.contrast_red      = xsane.contrast_red;
+      image_info.contrast_green    = xsane.contrast_green;
+      image_info.contrast_blue     = xsane.contrast_blue;
  
-      image_info.threshold        = xsane.threshold;
+      image_info.threshold         = xsane.threshold;
 
+      image_info.reduce_to_lineart = xsane.expand_lineart_to_grayscale;
 
       xsane_write_pnm_header(xsane.out, &image_info, 0);
     }
@@ -1114,7 +1127,7 @@ void xsane_scan_done(SANE_Status status)
           /* no temporary file */
           if (xsane_create_secure_file(xsane.dummy_filename)) /* remove possibly existing symbolic links for security */
           {
-           char buf[256];
+           char buf[TEXTBUFSIZE];
 
             snprintf(buf, sizeof(buf), "%s %s %s\n", ERR_DURING_SAVE, ERR_CREATE_SECURE_FILE, xsane.dummy_filename);
             xsane_back_gtk_error(buf, TRUE);
@@ -1136,7 +1149,7 @@ void xsane_scan_done(SANE_Status status)
           }
           else
           {
-           char buf[256];
+           char buf[TEXTBUFSIZE];
             DBG(DBG_info, "open of file `%s'failed : %s\n", xsane.dummy_filename, strerror(errno));
             snprintf(buf, sizeof(buf), "%s `%s': %s", ERR_OPEN_FAILED, xsane.dummy_filename, strerror(errno));
             xsane_back_gtk_error(buf, TRUE);
@@ -1154,7 +1167,7 @@ void xsane_scan_done(SANE_Status status)
       }
       else
       {
-       char buf[256];
+       char buf[TEXTBUFSIZE];
         DBG(DBG_info, "open of file `%s'failed : %s\n", xsane.dummy_filename, strerror(errno));
         snprintf(buf, sizeof(buf), "%s `%s': %s", ERR_OPEN_FAILED, xsane.dummy_filename, strerror(errno));
         xsane_back_gtk_error(buf, TRUE);
@@ -1174,7 +1187,20 @@ void xsane_scan_done(SANE_Status status)
 
     if (xsane.xsane_mode == XSANE_VIEWER)
     {
-      xsane_viewer_new(xsane.dummy_filename, xsane.expand_lineart_to_grayscale, 0);
+      if (!xsane.force_filename) /* user filename selection active */
+      {
+        xsane_viewer_new(xsane.dummy_filename, preferences.filetype, TRUE, preferences.filename, VIEWER_FULL_MODIFICATION, IMAGE_NOT_SAVED);
+      }
+      else
+      {
+        xsane_viewer_new(xsane.dummy_filename, NULL, TRUE, xsane.external_filename, VIEWER_NO_NAME_MODIFICATION, IMAGE_NOT_SAVED);
+      }
+
+      xsane.expand_lineart_to_grayscale = 0;
+    }
+
+    if ((xsane.xsane_mode == XSANE_MULTIPAGE) || (xsane.xsane_mode == XSANE_FAX) || (xsane.xsane_mode == XSANE_EMAIL))
+    {
       xsane.expand_lineart_to_grayscale = 0;
     }
 
@@ -1191,22 +1217,6 @@ void xsane_scan_done(SANE_Status status)
 
     if (xsane.xsane_mode == XSANE_SAVE)
     {
-      if (xsane.print_filenames) /* print created filenames to stdout? */
-      {
-        if (xsane.output_filename[0] != '/') /* relative path */
-        {
-         char pathname[512];
-          getcwd(pathname, sizeof(pathname));
-          printf("XSANE_IMAGE_FILENAME: %s/%s\n", pathname, xsane.output_filename);
-          fflush(stdout);
-        }
-        else /* absolute path */
-        {
-          printf("XSANE_IMAGE_FILENAME: %s\n", xsane.output_filename);
-          fflush(stdout);
-        }
-      }
-
       if ( ( (xsane.xsane_output_format != XSANE_PNM) && /* these files do not need any transformation */
              (xsane.xsane_output_format != XSANE_RGBA) ) ||
            (xsane.mode == XSANE_GIMP_EXTENSION) )
@@ -1228,7 +1238,7 @@ void xsane_scan_done(SANE_Status status)
         else
 #endif /* HAVE_ANY_GIMP */
         {
-          xsane_save_image_as(xsane.dummy_filename, xsane.output_filename, xsane.xsane_output_format, xsane.progress_bar, &xsane.cancel_save);
+          xsane_save_image_as(xsane.output_filename, xsane.dummy_filename, xsane.xsane_output_format, xsane.progress_bar, &xsane.cancel_save);
         }
 
         xsane_progress_clear();
@@ -1240,12 +1250,28 @@ void xsane_scan_done(SANE_Status status)
 
         remove(xsane.dummy_filename);
       }
+
+      if (xsane.print_filenames) /* print created filenames to stdout? */
+      {
+        if (xsane.output_filename[0] != '/') /* relative path */
+        {
+         char pathname[512];
+          getcwd(pathname, sizeof(pathname));
+          printf("XSANE_IMAGE_FILENAME: %s/%s\n", pathname, xsane.output_filename);
+          fflush(stdout);
+        }
+        else /* absolute path */
+        {
+          printf("XSANE_IMAGE_FILENAME: %s\n", xsane.output_filename);
+          fflush(stdout);
+        }
+      }
     }
     else if (xsane.xsane_mode == XSANE_COPY)
     {
      FILE *outfile;
      FILE *infile;
-     char buf[256];
+     char buf[TEXTBUFSIZE];
 
       DBG(DBG_info, "XSANE_COPY\n");
 
@@ -1270,14 +1296,13 @@ void xsane_scan_done(SANE_Status status)
                                             preferences.printer[preferences.printernr]->copy_number_option,
                                             xsane.copy_number);
       outfile = popen(buf, "w");
-/*      outfile = popen(preferences.printer[preferences.printernr]->command, "w"); */
       if ((outfile != 0) && (infile != 0)) /* copy mode, use zoom size */
       {
        struct SIGACTION act;
        float imagewidth, imageheight;
        int printer_resolution;
 
-         switch (xsane.param.format)
+        switch (xsane.param.format)
         {
           case SANE_FRAME_GRAY:
             if (xsane.depth == 1)
@@ -1301,15 +1326,16 @@ void xsane_scan_done(SANE_Status status)
 
         xsane_read_pnm_header(infile, &image_info);
 
-        imagewidth  = 72.0 * image_info.image_width/(float)printer_resolution; /* width in 1/72 inch */
-        imageheight = 72.0 * image_info.image_height/(float)printer_resolution; /* height in 1/72 inch */
+        imagewidth  = 72.0 * image_info.image_width /image_info.resolution_x * xsane.zoom; /* desired width in 1/72 inch */
+        imageheight = 72.0 * image_info.image_height/image_info.resolution_y * xsane.zoom; /* desired height in 1/72 inch */
 
         memset (&act, 0, sizeof (act)); /* define broken pipe handler */
         act.sa_handler = xsane_sigpipe_handler;
         sigaction (SIGPIPE, &act, 0);
 
-        DBG(DBG_info, "imagewidth  = %f\n 1/72 inch", imagewidth);
-        DBG(DBG_info, "imageheight = %f\n 1/72 inch", imageheight);
+        DBG(DBG_info, "imagewidth  = %f 1/72 inch\n", imagewidth);
+        DBG(DBG_info, "imageheight = %f 1/72 inch\n", imageheight);
+        DBG(DBG_info, "zoom        = %f\n", xsane.zoom);
 
         xsane_save_ps(outfile, infile,
                       &image_info,
@@ -1319,12 +1345,13 @@ void xsane_scan_done(SANE_Status status)
                       preferences.printer[preferences.printernr]->width  * 72.0/MM_PER_INCH, /* usable paperwidth */
                       preferences.printer[preferences.printernr]->height * 72.0/MM_PER_INCH, /* usable paperheight */
                       preferences.paper_orientation,
+                      preferences.printer[preferences.printernr]->ps_flatdecoded, /* ps level 3 */
                       xsane.progress_bar,
                       &xsane.cancel_save);
       }
       else
       {
-       char buf[256];
+       char buf[TEXTBUFSIZE];
 
         if (!infile)
         {
@@ -1360,122 +1387,8 @@ void xsane_scan_done(SANE_Status status)
         pclose(outfile);
       }
     }
-    else if (xsane.xsane_mode == XSANE_FAX)
-    {
-     FILE *outfile;
-     FILE *infile;
 
-       /* open progressbar */
-       xsane_progress_new(PROGRESS_CONVERTING_DATA, PROGRESS_TRANSFERING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
-
-       while (gtk_events_pending())
-       {
-         gtk_main_iteration();
-       }
-
-       infile = fopen(xsane.dummy_filename, "rb"); /* read binary (b for win32) */
-       if (infile != 0)
-       {
-         xsane_read_pnm_header(infile, &image_info);
-
-         umask((mode_t) preferences.image_umask); /* define image file permissions */   
-         outfile = fopen(xsane.fax_filename, "wb"); /* b = binary mode for win32 */
-         umask(XSANE_DEFAULT_UMASK); /* define new file permissions */   
-         if (outfile != 0)
-         {
-          float imagewidth, imageheight;
-
-           imagewidth  = 72.0 * image_info.image_width/xsane.resolution_x; /* width in 1/72 inch */
-           imageheight = 72.0 * image_info.image_height/xsane.resolution_y; /* height in 1/72 inch */
-
-           DBG(DBG_info, "imagewidth  = %f\n 1/72 inch", imagewidth);
-           DBG(DBG_info, "imageheight = %f\n 1/72 inch", imageheight);
-
-           xsane_save_ps(outfile, infile,
-                         &image_info,
-                         imagewidth, imageheight,
-                         preferences.fax_leftoffset   * 72.0/MM_PER_INCH, /* paper_left_margin */
-                         preferences.fax_bottomoffset * 72.0/MM_PER_INCH, /* paper_bottom_margin */
-                         preferences.fax_width  * 72.0/MM_PER_INCH, /* paper_width */
-                         preferences.fax_height * 72.0/MM_PER_INCH, /* paper_height */
-                         0 /* portrait top left */,
-                         xsane.progress_bar,
-                         &xsane.cancel_save);
-           fclose(outfile);
-         }
-         else
-         {
-          char buf[256];
-
-           DBG(DBG_info, "open of faxfile `%s'failed : %s\n", xsane.fax_filename, strerror(errno));
-
-           snprintf(buf, sizeof(buf), "%s `%s': %s", ERR_OPEN_FAILED, xsane.fax_filename, strerror(errno));
-           xsane_back_gtk_error(buf, TRUE);
-         }
-
-         fclose(infile);
-         remove(xsane.dummy_filename);
-       }
-       else
-       {
-        char buf[256];
-
-         DBG(DBG_info, "open of faxfile `%s'failed : %s\n", xsane.fax_filename, strerror(errno));
-
-         snprintf(buf, sizeof(buf), "%s `%s': %s", ERR_OPEN_FAILED, xsane.fax_filename, strerror(errno));
-         xsane_back_gtk_error(buf, TRUE);
-       }
-       xsane_progress_clear();
-
-       while (gtk_events_pending())
-       {
-         DBG(DBG_info, "calling gtk_main_iteration");
-         gtk_main_iteration();
-       }
-    }
-#ifdef XSANE_ACTIVATE_MAIL
-    else if (xsane.xsane_mode == XSANE_MAIL)
-    {
-     FILE *infile;
-
-       /* open progressbar */
-       xsane_progress_new(PROGRESS_CONVERTING_DATA, PROGRESS_TRANSFERING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
-
-       while (gtk_events_pending())
-       {
-         gtk_main_iteration();
-       }
-
-       infile = fopen(xsane.dummy_filename, "rb"); /* read binary (b for win32) */
-       if (infile != 0)
-       {
-         xsane_read_pnm_header(infile, &image_info);
-
-         xsane_save_image_as(xsane.dummy_filename, xsane.mail_filename, XSANE_PNG, xsane.progress_bar, &xsane.cancel_save);
-
-         fclose(infile);
-         remove(xsane.dummy_filename);
-       }
-       else
-       {
-        char buf[256];
-
-         DBG(DBG_info, "open of mailfile `%s'failed : %s\n", xsane.dummy_filename, strerror(errno));
-
-         snprintf(buf, sizeof(buf), "%s `%s': %s", ERR_OPEN_FAILED, xsane.dummy_filename, strerror(errno));
-         xsane_back_gtk_error(buf, TRUE);
-       }
-       xsane_progress_clear();
-
-       while (gtk_events_pending())
-       {
-         DBG(DBG_info, "calling gtk_main_iteration");
-         gtk_main_iteration();
-       }
-    }
-#endif
-
-    if ( (xsane.xsane_mode == XSANE_SAVE) && (xsane.mode == XSANE_STANDALONE) )
+    if ( ( (xsane.xsane_mode == XSANE_SAVE) || (xsane.xsane_mode == XSANE_VIEWER) ) && (xsane.mode == XSANE_STANDALONE) )
     {
       if (!xsane.force_filename) /* user filename selection active */
       {
@@ -1492,48 +1405,129 @@ void xsane_scan_done(SANE_Status status)
         xsane_update_counter_in_filename(&xsane.external_filename, TRUE, 1, 0);
       }
     }
+    else if (xsane.xsane_mode == XSANE_MULTIPAGE)
+    {
+     GtkWidget *list_item;
+     char *page;
+     char *type;
+     char *extension;
+
+      page = strdup(strrchr(xsane.multipage_filename,'/')+1);
+      extension = strrchr(page, '.');
+      if (extension)
+      {
+        type = strdup(extension);
+        *extension = 0;
+      }
+      else
+      {
+        type = "";
+      }
+
+      list_item = gtk_list_item_new_with_label(page);
+      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data", strdup(page));
+      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_type", strdup(type));
+      gtk_container_add(GTK_CONTAINER(xsane.project_list), list_item);
+      gtk_widget_show(list_item);
+
+      xsane_update_counter_in_filename(&xsane.multipage_filename, TRUE, 1, preferences.filename_counter_len);
+      xsane_multipage_project_save();
+      free(page);
+      free(type);
+
+      if (xsane.multipage_status)
+      {
+        free(xsane.multipage_status);
+      }
+      xsane.multipage_status = strdup(TEXT_PROJECT_STATUS_CHANGED);
+
+      xsane_multipage_project_save();
+
+      gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.multipage_status));
+      gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
+    }
     else if (xsane.xsane_mode == XSANE_FAX)
     {
      GtkWidget *list_item;
      char *page;
+     char *type;
      char *extension;
 
       page = strdup(strrchr(xsane.fax_filename,'/')+1);
       extension = strrchr(page, '.');
       if (extension)
       {
+        type = strdup(extension);
         *extension = 0;
       }
+      else
+      {
+        type = "";
+      }
+
       list_item = gtk_list_item_new_with_label(page);
       gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data", strdup(page));
-      gtk_container_add(GTK_CONTAINER(xsane.fax_list), list_item);
+      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_type", strdup(type));
+      gtk_container_add(GTK_CONTAINER(xsane.project_list), list_item);
       gtk_widget_show(list_item);
 
       xsane_update_counter_in_filename(&xsane.fax_filename, TRUE, 1, preferences.filename_counter_len);
       xsane_fax_project_save();
       free(page);
+      free(type);
+
+      if (xsane.fax_status)
+      {
+        free(xsane.fax_status);
+      }
+      xsane.fax_status = strdup(TEXT_PROJECT_STATUS_CHANGED);
+
+      xsane_fax_project_save();
+
+      gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.fax_status));
+      gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
     }
-#ifdef XSANE_ACTIVATE_MAIL
-    else if (xsane.xsane_mode == XSANE_MAIL)
+#ifdef XSANE_ACTIVATE_EMAIL
+    else if (xsane.xsane_mode == XSANE_EMAIL)
     {
      GtkWidget *list_item;
-     char *image;
+     char *page;
+     char *type;
      char *extension;
 
-      image = strdup(strrchr(xsane.mail_filename,'/')+1);
-      extension = strrchr(image, '.');
+      page = strdup(strrchr(xsane.email_filename,'/')+1);
+      extension = strrchr(page, '.');
       if (extension)
       {
+        type = strdup(extension);
         *extension = 0;
       }
-      list_item = gtk_list_item_new_with_label(image);
-      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data", strdup(image));
-      gtk_container_add(GTK_CONTAINER(xsane.mail_list), list_item);
+      else
+      {
+        type = "";
+      }
+
+      list_item = gtk_list_item_new_with_label(page);
+      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data", strdup(page));
+      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_type", strdup(type));
+      gtk_container_add(GTK_CONTAINER(xsane.project_list), list_item);
       gtk_widget_show(list_item);
 
-      xsane_update_counter_in_filename(&xsane.mail_filename, TRUE, 1, preferences.filename_counter_len);
-      xsane_mail_project_save();
-      free(image);
+      xsane_update_counter_in_filename(&xsane.email_filename, TRUE, 1, preferences.filename_counter_len);
+      xsane_email_project_save();
+      free(page);
+      free(type);
+
+      if (xsane.email_status)
+      {
+        free(xsane.email_status);
+      }
+      xsane.email_status = strdup(TEXT_PROJECT_STATUS_CHANGED);
+
+      xsane_email_project_save();
+
+      gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.email_status));
+      gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
     }
 #endif
   }
@@ -1566,15 +1560,16 @@ void xsane_scan_done(SANE_Status status)
 
     DBG(DBG_info, "ADF mode end of scan: increment page counter and restart scan\n");
     xsane.adf_page_counter += 1;
-    g_signal_emit_by_name(xsane.start_button, "clicked"); /* press START button */
+    gtk_timeout_add(100, (GtkFunction)xsane_scan_dialog, NULL); /* wait 100ms then call xsane_scan_dialog(); */
   }
-  else if ( ( (status == SANE_STATUS_GOOD) || (status == SANE_STATUS_EOF) ) && (xsane.batch_loop) )
+  else if ( ( (status == SANE_STATUS_GOOD) || (status == SANE_STATUS_EOF) ) && (xsane.batch_loop  == BATCH_MODE_LOOP) )
   {
     /* batch scan loop, this is not the last scan */
     DBG(DBG_info, "Batch mode end of scan\n");
     sane_cancel(xsane.dev); /* we have to call sane_cancel otherwise we are not able to set new parameters */
   }
-  else if ( ( (status != SANE_STATUS_GOOD) && (status != SANE_STATUS_EOF) ) || (!xsane.batch_loop) ) /* last scan: update histogram */
+//  else if ( ( (status != SANE_STATUS_GOOD) && (status != SANE_STATUS_EOF) ) || (!xsane.batch_loop) ) /* last scan: update histogram */
+  else
   {
     DBG(DBG_info, "Normal end of scan\n");
     xsane_set_sensitivity(TRUE);		/* reactivate buttons etc */
@@ -1622,7 +1617,7 @@ static void xsane_start_scan(void)
  SANE_Status status;
  SANE_Handle dev = xsane.dev;
  const char *frame_type = 0;
- char buf[256];
+ char buf[TEXTBUFSIZE];
  int fd;
  Image_info image_info;
 
@@ -1684,14 +1679,17 @@ static void xsane_start_scan(void)
     default:			frame_type = "unknown"; break;
   }
 
-  if ( (xsane.param.depth == 1) && (xsane.scan_rotation) )
+  if ((xsane.param.depth == 1) && ((xsane.scan_rotation) ||
+                                   (xsane.xsane_mode == XSANE_VIEWER) ||
+                                   (xsane.xsane_mode == XSANE_MULTIPAGE) ||
+                                   (xsane.xsane_mode == XSANE_FAX) ||
+                                   (xsane.xsane_mode == XSANE_EMAIL))
+     ) /* We want to do a transformation with a lineart scan */
+       /* or use the viewer to display a lineart scan, */
+       /* so we save it as grayscale */
   {
-    xsane.expand_lineart_to_grayscale = 1; /* We want to do transformation with lineart scan, so we save it as grayscale */
-  }
-
-  if ((xsane.xsane_mode == XSANE_VIEWER) && (xsane.param.depth == 1))
-  {
-    xsane.expand_lineart_to_grayscale = 1; /* we are using the viewer, lineart is not supported, so create grayscale */
+    DBG(DBG_proc, "lineart scan will be expanded to grayscale\n");
+    xsane.expand_lineart_to_grayscale = 1;
   }
 
   if (!xsane.header_size) /* first pass of multi pass scan or single pass scan */
@@ -1753,6 +1751,8 @@ static void xsane_start_scan(void)
  
     image_info.threshold        = xsane.threshold;
 
+    image_info.reduce_to_lineart = xsane.expand_lineart_to_grayscale;
+
     xsane_write_pnm_header(xsane.out, &image_info, 0);
 
     fflush(xsane.out);
@@ -1768,7 +1768,24 @@ static void xsane_start_scan(void)
   xsane.pixelcolor = 0;
 
   snprintf(buf, sizeof(buf), PROGRESS_RECEIVING_FRAME_DATA, _(frame_type));
-  xsane_progress_new(buf, PROGRESS_SCANNING, (GtkSignalFunc) xsane_cancel, NULL);
+  
+  if (preferences.adf_pages_max > 1)
+  {
+   char buf2[TEXTBUFSIZE];
+    if (preferences.adf_pages_max > 1)
+    {
+      snprintf(buf2, sizeof(buf2), "%s (%d/%d)", PROGRESS_SCANNING, xsane.adf_page_counter+1, preferences.adf_pages_max);
+    }
+    else
+    {
+      snprintf(buf2, sizeof(buf2), "%s (%d)", PROGRESS_SCANNING, xsane.adf_page_counter+1);
+    }
+    xsane_progress_new(buf, buf2, (GtkSignalFunc) xsane_cancel, NULL);
+  }
+  else
+  {
+    xsane_progress_new(buf, PROGRESS_SCANNING, (GtkSignalFunc) xsane_cancel, NULL);
+  }
 
   while (gtk_events_pending())
   {
@@ -1778,13 +1795,14 @@ static void xsane_start_scan(void)
   xsane.input_tag = -1;
 
   xsane.lineart_to_grayscale_x = xsane.param.pixels_per_line;
-
+#ifndef BUGGY_GDK_INPUT_EXCEPTION
   if ((sane_set_io_mode(dev, SANE_TRUE) == SANE_STATUS_GOOD) && (sane_get_select_fd(dev, &fd) == SANE_STATUS_GOOD))
   {
     DBG(DBG_info, "gdk_input_add\n");
     xsane.input_tag = gdk_input_add(fd, GDK_INPUT_READ | GDK_INPUT_EXCEPTION, xsane_read_image_data, 0);
   }
   else
+#endif
   {
     xsane_read_image_data(0, -1, GDK_INPUT_READ);
   }
@@ -1794,9 +1812,11 @@ static void xsane_start_scan(void)
 
 /* Invoked when the scan button is pressed */
 /* or by scan_done if automatic document feeder is selected */
-void xsane_scan_dialog(void)
+/* always returns 0 beacause ADF function calls it as timeout function */
+/* and return value 0 is used to tell the timeout handler to stop timer */
+gint xsane_scan_dialog(gpointer *data)
 {
- char buf[256];
+ char buf[TEXTBUFSIZE];
  const SANE_Option_Descriptor *opt;
 
   DBG(DBG_proc, "xsane_scan_dialog\n");
@@ -1807,12 +1827,13 @@ void xsane_scan_dialog(void)
 
   sane_get_parameters(xsane.dev, &xsane.param); /* update xsane.param */
 
-  if ( (xsane.mode == XSANE_STANDALONE) && (xsane.xsane_mode == XSANE_SAVE) )
+  if ( ( (xsane.xsane_mode == XSANE_SAVE) || (xsane.xsane_mode == XSANE_VIEWER) ) && (xsane.mode == XSANE_STANDALONE) )
   {
     /* correct length of filename counter if it is shorter than minimum length */
     if (!xsane.force_filename)
     {
-      if (xsane.batch_loop) /* we are doing a batch scan, so we need a counter in the filename */
+      /* when we are doing an adf or batch scan then we need a counter in the filename */
+      if ((xsane.batch_loop) || (preferences.adf_pages_max > 1))
       {
         xsane_ensure_counter_in_filename(&preferences.filename, preferences.filename_counter_len);
       }
@@ -1835,7 +1856,7 @@ void xsane_scan_dialog(void)
       testfile = fopen(xsane.output_filename, "rb"); /* read binary (b for win32) */
       if (testfile) /* filename used: skip */
       {
-       char buf[256];
+       char buf[TEXTBUFSIZE];
 
         fclose(testfile);
 
@@ -1843,7 +1864,7 @@ void xsane_scan_dialog(void)
         if (xsane_back_gtk_decision(ERR_HEADER_WARNING, (gchar **) warning_xpm, buf, BUTTON_OVERWRITE, BUTTON_CANCEL, TRUE /* wait */) == FALSE)
         {
           xsane_set_sensitivity(TRUE);
-          return;
+          return FALSE;
         }
       }
     }
@@ -1864,7 +1885,7 @@ void xsane_scan_dialog(void)
         }
         xsane_back_gtk_error(buf, TRUE);
         xsane_set_sensitivity(TRUE);
-       return;
+       return FALSE;
       }
 #if 0 /* this is not necessary any more because the saving routines do the conversion */
       else if ( ( ( (xsane.xsane_output_format == XSANE_JPEG) && xsane.param.depth == 16) ||
@@ -1876,7 +1897,7 @@ void xsane_scan_dialog(void)
         if (xsane_back_gtk_decision(ERR_HEADER_INFO, (gchar **) info_xpm, buf, BUTTON_REDUCE, BUTTON_CANCEL, TRUE /* wait */) == FALSE)
         {
           xsane_set_sensitivity(TRUE);
-         return;
+         return FALSE;
         }
         xsane.reduce_16bit_to_8bit = TRUE;
       }
@@ -1888,7 +1909,7 @@ void xsane_scan_dialog(void)
         snprintf(buf, sizeof(buf), "No RGBA data format !!!"); /* user selected output format RGBA, scanner uses other format */
         xsane_back_gtk_error(buf, TRUE);
         xsane_set_sensitivity(TRUE);
-       return;
+       return FALSE;
       }
 #endif
     }
@@ -1898,7 +1919,7 @@ void xsane_scan_dialog(void)
       snprintf(buf, sizeof(buf), "Special format RGBA only supported in scan mode !!!");
       xsane_back_gtk_error(buf, TRUE);
       xsane_set_sensitivity(TRUE);
-     return;
+     return FALSE;
     }
 #endif
 
@@ -1910,7 +1931,7 @@ void xsane_scan_dialog(void)
         snprintf(buf, sizeof(buf), "Image data of type SANE_FRAME_RGBA\ncan only be saved in rgba or png format");
         xsane_back_gtk_error(buf, TRUE);
         xsane_set_sensitivity(TRUE);
-       return;
+       return FALSE;
       }
     }
 #endif
@@ -1938,7 +1959,7 @@ void xsane_scan_dialog(void)
           if (xsane_back_gtk_decision(ERR_HEADER_INFO, (gchar **) info_xpm, buf, BUTTON_REDUCE, BUTTON_CANCEL, TRUE /* wait */) == FALSE)
           {
             xsane_set_sensitivity(TRUE);
-           return;
+           return FALSE;
           }
           xsane.reduce_16bit_to_8bit = TRUE;
         }
@@ -1947,7 +1968,7 @@ void xsane_scan_dialog(void)
           snprintf(buf, sizeof(buf), ERR_GIMP_BAD_DEPTH, xsane.param.depth);
           xsane_back_gtk_error(buf, TRUE);
           xsane_set_sensitivity(TRUE);
-         return;
+         return FALSE;
         }
       }
     }
@@ -2112,6 +2133,7 @@ void xsane_scan_dialog(void)
   xsane.scanning = TRUE; /* set marker that scan has been initiated */
 
   xsane_start_scan();
+ return FALSE;
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
