@@ -26,8 +26,12 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+#ifndef XSANE_SETUP_H
+# define XSANE_SETUP_H
+
 extern void xsane_new_printer(void);   
 extern void xsane_update_int(GtkWidget *widget, int *val);
 extern void xsane_setup_dialog(GtkWidget *widget, gpointer data);
 
+#endif
 /* ---------------------------------------------------------------------------------------------------------------------- */
